package defi

import (
	"bytes"
	"encoding/json"
	"net/http"
	"net/url"
	"os"
	"strconv"
	"strings"

	"birdcall/internal/library/birdeye"

	"github.com/gogf/gf/v2/util/gconv"
)

type Defi struct{}

// 获取当前支持的网络
// @return *birdeye.CommonRes
// @return error
func (d *Defi) Support() (*NetWorkRes, error) {
	//构建基本请求连接
	baseURL := os.Getenv(birdeye.DEFI_URL)
	fullURL := baseURL + birdeye.NET_WORKS

	apiReq, err := http.NewRequest(birdeye.GET, fullURL, nil)
	if err != nil {
		return nil, err
	}
	//构建并发送请求
	apiReq = birdeye.MakeHeader(apiReq)
	body, err := birdeye.SendRequest(apiReq)
	if err != nil {
		return nil, err
	}

	//处理返回数据
	var res *NetWorkRes
	err = gconv.Struct(body, &res)
	if err != nil {
		return nil, err
	}
	return res, nil
}

//	 Price 获取token的价格
//		@param check_liquidity
//		@param include_liquidity
//		@param address
//		@return *birdeye.CommonRes
//		@return error
func (d *Defi) Price(chain_name string, check_liquidity float64, include_liquidity bool, address string) (*PriceRes, error) {
	//构建基本请求连接
	baseURL := os.Getenv(birdeye.DEFI_URL)
	params := url.Values{}
	params.Add(birdeye.CheckLiquidity, strconv.FormatFloat(check_liquidity, 'f', 2, 64))
	params.Add(birdeye.IncludeLiquidity, strconv.FormatBool(include_liquidity))
	params.Add(birdeye.ADDRESS, address)
	fullURL := baseURL + birdeye.PRICE + birdeye.QUESTION_MARK + params.Encode()

	apiReq, err := http.NewRequest(birdeye.GET, fullURL, nil)
	if err != nil {
		return nil, err
	}

	//构建请求头
	apiReq = birdeye.MakeHeader(apiReq, chain_name)

	body, err := birdeye.SendRequest(apiReq)
	if err != nil {
		return nil, err
	}
	//处理返回数据
	var res *PriceRes
	err = gconv.Struct(body, &res)
	if err != nil {
		return nil, err
	}
	return res, nil
}

// dc：一次获取多个token的价格
// @return *birdeye.CommonRes
// @return error
func (d *Defi) PriceMultiple(chain_name string, check_liquidity float64, include_liquidity bool, address []string, method string) (*MultiplePriceRes, error) {
	//构建基本请求连接
	baseURL := os.Getenv(birdeye.DEFI_URL)
	params := url.Values{}
	params.Add(birdeye.CheckLiquidity, strconv.FormatFloat(check_liquidity, 'f', 2, 64))
	params.Add(birdeye.IncludeLiquidity, strconv.FormatBool(include_liquidity))

	var ApiReq *http.Request
	var payload *bytes.Buffer = nil

	//判断请求类型
	if method == birdeye.GET {
		addresses := strings.Join(address, birdeye.COMMAS)
		params.Add(birdeye.LIST_ADDRESS, addresses)
	} else if method == birdeye.POST {
		data, err := json.Marshal(address)
		if err != nil {
			return nil, err
		}
		payload = bytes.NewBuffer(data)
	}

	fullURL := baseURL + birdeye.MULTIPLE_PRICE + birdeye.QUESTION_MARK + params.Encode()

	if method == birdeye.GET {
		apiReq, err := http.NewRequest(method, fullURL, nil)
		if err != nil {
			return nil, err
		}
		ApiReq = apiReq
	} else if method == birdeye.POST {
		apiReq, err := http.NewRequest(method, fullURL, payload)
		if err != nil {
			return nil, err
		}
		ApiReq = apiReq
	}

	//构建请求头
	ApiReq = birdeye.MakeHeader(ApiReq, chain_name)

	//发送请求
	body, err := birdeye.SendRequest(ApiReq)
	if err != nil {
		return nil, err
	}
	var res *MultiplePriceRes
	err = gconv.Struct(body, &res)
	if err != nil {
		return nil, err
	}
	return res, nil
}

//	 PriceHistorical
//		@param chain_name
//		@param address_type
//		@param time_type
//		@param time_from
//		@param time_to
//		@param address
//		@return *PriceHistoricalRes
//		@return error
func (d *Defi) PriceHistorical(chain_name string, address_type string, time_type string, time_from uint64, time_to uint64, address string) (*PriceHistoricalRes, error) {
	//构建基本请求连接
	baseURL := os.Getenv(birdeye.DEFI_URL)
	params := url.Values{}
	params.Add(birdeye.ADDRESS, address)
	params.Add(birdeye.ADDRESS_TYPE, address_type)
	params.Add(birdeye.TYPE, time_type)
	params.Add(birdeye.TIME_FROM, strconv.FormatUint(time_from, 10))
	params.Add(birdeye.TIME_TO, strconv.FormatUint(time_to, 10))
	fullURL := baseURL + birdeye.HISTORICAL + birdeye.QUESTION_MARK + params.Encode()

	apiReq, err := http.NewRequest(birdeye.GET, fullURL, nil)
	if err != nil {
		return nil, err
	}

	//构建请求头
	apiReq = birdeye.MakeHeader(apiReq, chain_name)

	body, err := birdeye.SendRequest(apiReq)
	if err != nil {
		return nil, err
	}
	//处理返回数据
	var res *PriceHistoricalRes
	err = gconv.Struct(body, &res)
	if err != nil {
		return nil, err
	}
	return res, nil
}

//	 PriceHistoricalUnix
//		@param chain_name
//		@param address
//		@param unix_time
//		@return *PriceHistoricalUnixRes
//		@return error
func (d *Defi) PriceHistoricalUnix(chain_name string, address string, unix_time uint64) (*PriceHistoricalUnixRes, error) {
	//构建基本请求连接
	baseURL := os.Getenv(birdeye.DEFI_URL)
	params := url.Values{}
	params.Add(birdeye.ADDRESS, address)
	params.Add(birdeye.UNIX_TIME, strconv.FormatUint(unix_time, birdeye.TEN))

	fullURL := baseURL + birdeye.HISTORICAL + birdeye.QUESTION_MARK + params.Encode()

	apiReq, err := http.NewRequest(birdeye.GET, fullURL, nil)
	if err != nil {
		return nil, err
	}

	//构建请求头
	apiReq = birdeye.MakeHeader(apiReq, chain_name)

	body, err := birdeye.SendRequest(apiReq)
	if err != nil {
		return nil, err
	}
	//处理返回数据
	var res *PriceHistoricalUnixRes
	err = gconv.Struct(body, &res)
	if err != nil {
		return nil, err
	}
	return res, nil
}

func (d *Defi) TradesToken(chain_name string, address string, offset uint64, limit uint64, tx_type string, sort_type string) (*TradeTokenRes, error) {
	//构建基本请求连接
	baseURL := os.Getenv(birdeye.DEFI_URL)
	params := url.Values{}
	params.Add(birdeye.ADDRESS, address)
	params.Add(birdeye.OFFSET, strconv.FormatUint(offset, birdeye.TEN))
	params.Add(birdeye.LIMIT, strconv.FormatUint(limit, birdeye.TEN))
	params.Add(birdeye.SORT_TYPE, sort_type)

	fullURL := baseURL + birdeye.TRADES_TOKEN + birdeye.QUESTION_MARK + params.Encode()

	apiReq, err := http.NewRequest(birdeye.GET, fullURL, nil)
	if err != nil {
		return nil, err
	}

	//构建请求头
	apiReq = birdeye.MakeHeader(apiReq, chain_name)

	body, err := birdeye.SendRequest(apiReq)
	if err != nil {
		return nil, err
	}
	//处理返回数据
	var res *TradeTokenRes
	err = gconv.Struct(body, &res)
	if err != nil {
		return nil, err
	}
	return res, nil
}

func (d *Defi) TradesPair(chain_name string, address string, offset uint64, limit uint64, tx_type string, sort_type string) (*TradePairRes, error) {
	//构建基本请求连接
	baseURL := os.Getenv(birdeye.DEFI_URL)
	params := url.Values{}
	params.Add(birdeye.ADDRESS, address)
	params.Add(birdeye.OFFSET, strconv.FormatUint(offset, birdeye.TEN))
	params.Add(birdeye.LIMIT, strconv.FormatUint(limit, birdeye.TEN))
	params.Add(birdeye.SORT_TYPE, sort_type)

	fullURL := baseURL + birdeye.TRADES_PAIR + birdeye.QUESTION_MARK + params.Encode()

	apiReq, err := http.NewRequest(birdeye.GET, fullURL, nil)
	if err != nil {
		return nil, err
	}

	//构建请求头
	apiReq = birdeye.MakeHeader(apiReq, chain_name)

	body, err := birdeye.SendRequest(apiReq)
	if err != nil {
		return nil, err
	}
	//处理返回数据
	var res *TradePairRes
	err = gconv.Struct(body, &res)
	if err != nil {
		return nil, err
	}
	return res, nil
}
